/**************************************************************
  
                 aicdrvr.c
  
                 Leor Brenman
  
                 03-23-92
  
           (C) Texas Instruments Inc., 1992 
  
           Refer to the file 'license.txt' included with this 
           this package for usage and license information. 
  
*************************************************************/
/**************************************************************************/
/*   AICDRVR.C                                                            */
/*                                                                        */
/*   TMS320C3x - AIC DRIVER                                               */
/*      :TMS320C3x CODE                                                   */
/*       Compile and archive into aic.lib                                 */
/*                                                                        */
/*   Leor Brenman - Applications Engineer                                 */
/*   (C) 1991 TEXAS INSTRUMENTS, HOUSTON                                  */
/**************************************************************************/
#include <math.h>
#include <stdlib.h>
#include <aic.h>

/**************************************************************************/
/* GLOBAL VARIABLES                                                       */
/**************************************************************************/
int t_buffer = BLOCK_SIZE;     /* SIZE OF I/O BUFFER(S)                   */
int r_buffer = BLOCK_SIZE;     /* SIZE OF I/O BUFFER(S)                   */
VPVF output;                   /* OUTPUT DATA BUFFER FOR PROCESSOR        */
VPVF input;                    /* INPUT DATA BUFFER FOR PROCESSOR         */
VPVF output_xfer;              /* OUTPUT DATA BUFFER FOR ISR/AIC          */
VPVF input_xfer;               /* INPUT DATA BUFFER FOR ISR/AIC           */
VI   buffer_rcvd = FALSE;      /* CPU-ISR COMM FLAG (INPUT)               */
VI   buffer_xmtd = FALSE;      /* CPU-ISR COMM FLAG (OUTPUT)              */
VI   r_index = 0;              /* INDEX INTO INPUT AND OUTPUT DATA ARRAYS */
VI   t_index = 0;              /* INDEX INTO INPUT AND OUTPUT DATA ARRAYS */
VI   i;                        /* GENERIC COUNTER VARIABLE                */

/**************************************************************************/
/* AIC CONTROL VARIABLES						  */
/**************************************************************************/
AIC_COMMAND_0           aic_command_0;              /* AIC COMMAND WORD 0 */
AIC_COMMAND_1           aic_command_1;              /* AIC COMMAND WORD 1 */
AIC_COMMAND_2           aic_command_2;              /* AIC COMMAND WORD 2 */
AIC_COMMAND_3           aic_command_3;              /* AIC COMMAND WORD 3 */
volatile AIC_PRIMARY	aic_primary;
VI  secondary_transmit = OFF;           /* FLAG TO SENT SECONDARY TRANSMIT*/
VI  aic_secondary = 0;              /* COMMAND SENT ON SECONDARY TRANSMIT */
int ie;                               /* ENABLED INTERRUPTS TEMP VARIABLE */

#if ASYNC
/**************************************************************************/
/* C_INT05() OR C_INT07()                                                 */
/*        SERIAL PORT 0/1 TRANSMIT INTERRUPT SERVICE ROUTINE              */
/*        1. IF SECONDARY TRANSMISSION SEND AIC COMMAND WORD              */
/*        2. OTHERWISE IF COMMAND SEND REQUESTED SETUP FOR SECONDARY      */
/*           TRANSMISSION ON NEXT INTERRUPT                               */
/*        3. OTHERWISE WRITE OUT OUTPUT DATA                              */
/*        4. RESET SAMPLE INDEX AND FLAG IF FRAME IS FULL                 */
/*           AND SWAP BUFFER POINTERS                                     */
/*        5. IF REAL TIME IS NOT MET GO TO ERROR HANDLER                  */
/**************************************************************************/
#if SER_NUM
void c_int05(void) {}
void c_int07(void)
#else
void c_int07(void) {}
void c_int05(void)
#endif
{
    VPVF swap;

    if (secondary_transmit)
    {
        SERIAL_PORT_ADDR(SER_NUM)->x_data = aic_secondary;
        secondary_transmit     = OFF;
        put_ie(ie);                        /* RESTORE GET ENABLED INTS    */
    }
    else
    {
        if (aic_primary._bitval.command == SECONDARY_REQ)
	{
             ie = get_ie();                        /* GET ENABLED INTS    */
             if(SER_NUM)
                 put_ie(0x40);                 /* ENABLE SERIAL PORT INT  */
	     else
		 put_ie(0x10);
	     secondary_transmit     = ON;
	}

        aic_primary._bitval.data = output_xfer[t_index];
        SERIAL_PORT_ADDR(SER_NUM)->x_data = aic_primary._intval;

	if (++t_index == t_buffer)
	{
#if ERROR_CHECK
            if(buffer_xmtd == TRUE) error_in_real_time();
#endif
            swap         = output_xfer;
	    output_xfer  = output;
            output       = swap;
	    t_index = 0;
	    buffer_xmtd = TRUE;
	}
    }
    aic_primary._bitval.command = STANDARD;
}

/**************************************************************************/
/* C_INT06() OR C_INT08()                                                 */
/*        SERIAL PORT 0/1 RECEIVE INTERRUPT SERVICE ROUTINE               */
/*        1. READ INPUT DATA                                              */
/*        2. RESET SAMPLE INDEX AND FLAG IF FRAME IS FULL                 */
/*           AND SWAP BUFFER POINTERS                                     */
/*        3. IF REAL TIME IS NOT MET GO TO ERROR HANDLER                  */
/**************************************************************************/
#if SER_NUM
void c_int06(void) {}
void c_int08(void)
#else
void c_int08(void) {}
void c_int06(void)
#endif
{
    VPVF swap;

    aic_primary._intval = SERIAL_PORT_ADDR(SER_NUM)->r_data & 0x0FFFF;
    input_xfer[r_index] = aic_primary._bitval.data;

    if (++r_index == r_buffer)
    {
#if ERROR_CHECK
        if(buffer_rcvd == TRUE) error_in_real_time();
#endif
        swap         = input;
	input	     = input_xfer;
        input_xfer   = swap;
	r_index = 0;
	buffer_rcvd = TRUE;
    }
}
#else /* IF NOT ASYNC */

/**************************************************************************/
/* C_INT05() OR C_INT07()                                                 */
/*        SERIAL PORT 0/1 TRANSMIT AND RECEIVE INTERRUPT SERVICE ROUTINE  */
/*        1. IF SECONDARY TRANSMISSION SEND AIC COMMAND WORD              */
/*        2. OTHERWISE IF COMMAND SEND REQUESTED SETUP FOR SECONDARY      */
/*           TRANSMISSION ON NEXT INTERRUPT AND RECEIVE DATA              */
/*        3. OTHERWISE WRITE OUT OUTPUT DATA AND RECEIVE INPUT DATA       */
/*        4. RESET SAMPLE INDEX AND FLAG IF FRAME IS FULL                 */
/*           AND SWAP BUFFER POINTERS                                     */
/*        5. IF REAL TIME IS NOT MET GO TO ERROR HANDLER                  */
/**************************************************************************/
#if SER_NUM
void c_int05(void) {}
void c_int06(void) {}
void c_int08(void) {}
void c_int07(void) {}
#else
void c_int06(void) {}
void c_int07(void) {}
void c_int08(void) {}
void c_int05(void)
#endif
{
    VPVF swap;

    if (secondary_transmit)
    {
        SERIAL_PORT_ADDR(SER_NUM)->x_data = aic_secondary;
        secondary_transmit     = OFF;
        put_ie(ie);                        /* RESTORE GET ENABLED INTS    */
    }
    else
    {
        if (aic_primary._bitval.command == SECONDARY_REQ)
	{
             ie = get_ie();                        /* GET ENABLED INTS    */
             if(SER_NUM)
                 put_ie(0x40);                 /* ENABLE SERIAL PORT INT  */
	     else
		 put_ie(0x10);
	     secondary_transmit     = ON;
	}

        aic_primary._bitval.data = output_xfer[t_index];
        SERIAL_PORT_ADDR(SER_NUM)->x_data = aic_primary._intval;

        aic_primary._intval = SERIAL_PORT_ADDR(SER_NUM)->r_data & 0x0FFFF;
        input_xfer[r_index] = aic_primary._bitval.data;

	if (++r_index == r_buffer)
	{
#if ERROR_CHECK
             if(buffer_rcvd == TRUE) error_in_real_time();
#endif
             swap         = input;
	     input	  = input_xfer;
             input_xfer   = swap;
	     r_index = 0;
	     buffer_rcvd = TRUE;
	}
	if (++t_index == t_buffer)
	{
#if ERROR_CHECK
            if(buffer_xmtd == TRUE) error_in_real_time();
#endif
            swap         = output_xfer;
	    output_xfer  = output;
            output       = swap;
	    t_index = 0;
	    buffer_xmtd = TRUE;
	}
    }
    aic_primary._bitval.command = STANDARD;
}
#endif

/**************************************************************************/
/* INIT_ARRAYS(): INITIALIZE DATA ARRAY PARAMETERS                        */
/**************************************************************************/
void init_arrays(int t_buffer, int r_buffer)
{
     int i;
     /*-------------------------------------------------------------------*/
     /* INITIALIZE AND ZERO FILL ARRAYS                                   */
     /*-------------------------------------------------------------------*/
     if(!(input       = (float *) malloc(r_buffer))) heap_overflow();
     if(!(output      = (float *) malloc(r_buffer))) heap_overflow();
     if(!(input_xfer  = (float *) malloc(r_buffer))) heap_overflow();
     if(!(output_xfer = (float *) malloc(r_buffer))) heap_overflow();
     for(i = 0; i < t_buffer; i++) output[i] = output_xfer[i] = 0.0;
}

/**************************************************************************/
/* INIT_AIC(): INITIALIZE COMMUNICATIONS TO AIC                           */
/*             NOTE: i IS A VOLATILE TO FORCE TIME DELAYS AND TO FORCE    */
/*                  READS OF SERIAL PORT DATA RECEIVE REGISTER TO CLEAR   */
/*                  THE RECEIVE INTERRUPT FLAG                            */
/**************************************************************************/
void init_aic(void)
{
    RESET_AIC;                                               /* RESET AIC */
    WAIT(50);                   /* KEEP RESET LOW FOR SOME PERIOD OF TIME */

    /*--------------------------------------------------------------------*/
    /* SET AIC CONFIGURATION CHIP                                         */
    /*  1. ALLOW 8 KHZ SAMPLING RATE AND 3.6 KHZ ANTIALIASING FILTER      */
    /*     GIVEN A 7.5 MHZ MCLK TO THE AIC FROM A 30 MHZ TMS320C30        */
    /*  2. ENABLE A/D HIGHPASS FILTER                                     */
    /*	3. SET SYNCHRONOUS TRANSMIT AND RECEIVE IF NOT ASYNC		  */
    /*  4. ENABLE SINX/X D/A CORRECTION FILTER                            */
    /*  5. SET AIC FOR +/- 1.5 V INPUT                                    */
    /*--------------------------------------------------------------------*/
    aic_primary._bitval.command    = STANDARD;
    aic_primary._bitval.data       = 0;
    aic_primary._bitval.unused     = 0;

#if TLC32046
    aic_command_0.command  = 0;       /* SETUP AIC COMMAND WORD ZERO      */
    aic_command_0.ra       = 26;      /* ADJUST SAMPLING RATE TO 8 kHz    */
    aic_command_0.ta       = 26;      /*    AND 3.6 kHz ANTIALIAS FILTER  */

    aic_command_1.command  = 1;       /* SETUP DEFAULT AIC COMMAND WORD 1 */
    aic_command_1.ra_prime = 1;
    aic_command_1.ta_prime = 1;
    aic_command_1.d_f      = 0;

    aic_command_2.command  = 2;       /* SETUP DEFAULT AIC COMMAND WORD 2 */
    aic_command_2.rb	   = 18;
    aic_command_2.tb	   = 18;

    aic_command_3.command  = 3;
    aic_command_3.highpass = ON;      /* TURN ON INPUT HIGHPASS FILTER	  */
    aic_command_3.loopback = OFF;     /* DISABLE AIC LOOPBACK             */
    aic_command_3.aux      = OFF;     /* DISABLE AUX INPUT                */
#if ASYNC
    aic_command_3.sync	   = OFF;     /* DISABLE SYNCHRONOUS A/D AND D/A  */
#else
    aic_command_3.sync	   = ON;      /* ENABLE SYNCHRONOUS A/D AND D/A   */
#endif
    aic_command_3.gain     = THREE_V; /* SET FOR LINE-LEVEL INPUT         */
    aic_command_3.d_8	   = 0;
    aic_command_3.sinx	   = ON;      /* ENABLE SIN x/x CORRECTION FILTER */
    aic_command_3.d10out   = OFF;     /* DISABLE D10OUT (TEL-I/F MODE)	  */
    aic_command_3.d11out   = OFF;     /* DISABLE D11OUT (TEL-I/F MODE)	  */
    aic_command_3.d_cdef   = 0;
#else
    aic_command_0.command  = 0;       /* SETUP AIC COMMAND WORD ZERO      */
    aic_command_0.ra	   = 13;      /* ADJUST SAMPLING RATE TO 8 kHz	  */
    aic_command_0.ta	   = 13;      /*    AND 3.6 kHz ANTIALIAS FILTER  */

    aic_command_1.command  = 1;       /* SETUP DEFAULT AIC COMMAND WORD 1 */
    aic_command_1.ra_prime = 1;
    aic_command_1.ta_prime = 1;
    aic_command_1.d_f      = 0;

    aic_command_2.command  = 2;       /* SETUP DEFAULT AIC COMMAND WORD 2 */
    aic_command_2.rb	   = 36;
    aic_command_2.tb	   = 36;

    aic_command_3.command  = 3;
    aic_command_3.highpass = ON;      /* TURN ON INPUT HIGHPASS FILTER	  */
    aic_command_3.loopback = OFF;     /* DISABLE AIC LOOPBACK             */
    aic_command_3.aux      = OFF;     /* DISABLE AUX INPUT                */
#if ASYNC
    aic_command_3.sync	   = OFF;     /* DISABLE SYNCHRONOUS A/D AND D/A  */
#else
    aic_command_3.sync	   = ON;      /* ENABLE SYNCHRONOUS A/D AND D/A   */
#endif
    aic_command_3.gain     = LINE_V;  /* SET FOR LINE-LEVEL INPUT         */
    aic_command_3.d_8	   = 0;
    aic_command_3.sinx	   = ON;      /* ENABLE SIN x/x CORRECTION FILTER */
    aic_command_3.d_abcdef   = 0;
#endif

#if MSTR_CLOCK
    /*--------------------------------------------------------------------*/
    /* CONFIGURE TIMER 0 TO ACT AS AIC MCLK                               */
    /* THE TIMER IS CONFIGURED IN THE FOLLOWING WAY                       */
    /*         1. THE TIMER'S VALUE DRIVES AN ACTIVE-HIGH TCLK 0 PIN      */
    /*         2. THE TIMER IS RESET AND ENABLED                          */
    /*         3. THE TIMER'S IS IN PULSE MODE                            */
    /*         4. THE TIMER HAS A PERIOD OF TWO INSTRUCTION CYCLES        */
    /*--------------------------------------------------------------------*/
    TIMER_ADDR(TIMER_NUM)->period = 1;
    TIMER_ADDR(TIMER_NUM)->gcontrol = FUNC | HLD_ | GO | CLKSRC;
#endif

    /*--------------------------------------------------------------------*/
    /* CONFIGURE SERIAL PORT 0                                            */
    /*            1. EXTERNAL FSX, FSR, CLKX, CLKR                        */
    /*            2. VARIABLE DATA RATE TRANSMIT AND RECEIVE              */
    /*            3. HANDSHAKE DISABLED                                   */
    /*            4. ACTIVE HIGH DATA AND CLK                             */
    /*            5. ACTIVE LOW FSX,FSR                                   */
    /*            6. 16 BIT TRANSMIT AND RECEIVE WORD                     */
    /*            7. TRANSMIT INTERRUPT                                   */
    /*            8. RECEIVE INTERRUPT ENABLED/RECEIVE                    */
    /*            9. FSX, FSR, CLKX, CLKR, DX, DR CONFIGURED AS SERIAL    */
    /*               PORT PINS                                            */
    /*--------------------------------------------------------------------*/
    SERIAL_PORT_ADDR(SER_NUM)->gcontrol = 0x0;

    SERIAL_PORT_ADDR(SER_NUM)->s_x_control = CLKXFUNC | DXFUNC | FSXFUNC;
    SERIAL_PORT_ADDR(SER_NUM)->s_r_control = CLKRFUNC | DRFUNC | FSRFUNC;

    SERIAL_PORT_ADDR(SER_NUM)->gcontrol = XVAREN | RVAREN | FSXP | FSRP |
                                          XLEN_16 | RLEN_16 | XINT | RINT |
                                          RRESET | XRESET;

    /* CLEAR SERIAL TRANSMIT DATA    */
    SERIAL_PORT_ADDR(SER_NUM)->x_data = 0x0;

    UN_RESET_AIC;
    CL_INT_FL_REG;

#if SER_NUM
    EN_SER_PORT_XMT_INT_1;
#else
    EN_SER_PORT_XMT_INT_0;
#endif

#if ASYNC
#if SER_NUM
    EN_SER_PORT_RCV_INT_1;
#else
    EN_SER_PORT_RCV_INT_0;
#endif
#endif

    EN_GLOBAL_INTS;	  /* SET GLOBAL INTERRUPT ENABLE BIT		  */
    /*--------------------------------------------------------------------*/
    /* MODIFY AIC CONFIGURATION                                           */
    /*--------------------------------------------------------------------*/
    configure_aic(*((int *) &aic_command_0));
    configure_aic(*((int *) &aic_command_3));
}

/**************************************************************************/
/* CONFIGURE_AIC(): INITIATE AIC CONFIGURATION WORD TRANSMISSION ON NEXT  */
/*                  INTERRUPT AFTER ALL PREVIOUS COMMANDS ARE SENT        */
/**************************************************************************/
void configure_aic(int i)
{
  while((aic_primary._bitval.command==SECONDARY_REQ) || secondary_transmit);
  aic_secondary = i;
  aic_primary._bitval.command = SECONDARY_REQ;
}

/**************************************************************************/
/* GET_IE(): read ie register                                             */
/**************************************************************************/
int get_ie(void)
{
   asm(" LDI   IE,R0");
}

/**************************************************************************/
/* PUT_IE(): write ie register                                            */
/**************************************************************************/
void put_ie(int ie_value)
{
#if _REGPARM
   asm(" LDI   AR2,IE");
#else
   asm(" LDI   *-FP(2),IE");
#endif /* _REGPARM */
}
